@@ -389,10 +389,30 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 				// start a thread that will associate this dataset with the group
 				if(toCreate.getChosenProfile() != null){
 
-					AssociationToGroupThread thread = new AssociationToGroupThread(toCreate.getChosenProfile(), datasetId, userName, utils, organizationNameOrId);
-					thread.start();
+					AssociationToGroupThread threadAssociationToGroup = 
+							new AssociationToGroupThread(
+									toCreate.getChosenProfile(), 
+									datasetId, 
+									userName, 
+									utils, 
+									organizationNameOrId
+									);
+					threadAssociationToGroup.start();
 
 				}
+				
+				// launch notification thread
+				WritePostCatalogueManagerThread threadWritePost = 
+						new WritePostCatalogueManagerThread(
+								userName,
+								scope, 
+								toCreate.getTitle(), 
+								datasetUrl, 
+								true, // send notification to other people 
+								toCreate.getTags(), 
+								toCreate.getAuthorFullName()
+								);
+				threadWritePost.start();
 
 				return toCreate;
 
