@@ -1,5 +1,8 @@
 package org.gcube.portlets.widgets.ckandatapublisherwidget.server.utils;
 
+import java.io.BufferedReader;
+import java.io.InputStream;
+import java.io.InputStreamReader;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
@@ -7,7 +10,14 @@ import java.util.Map;
 
 import javax.servlet.http.HttpSession;
 
+import org.apache.http.HttpEntity;
+import org.apache.http.HttpResponse;
+import org.apache.http.client.methods.HttpPost;
+import org.apache.http.entity.StringEntity;
+import org.apache.http.impl.client.CloseableHttpClient;
+import org.apache.http.impl.client.HttpClientBuilder;
 import org.gcube.application.framework.core.session.ASLSession;
+import org.gcube.common.authorization.library.provider.SecurityTokenProvider;
 import org.gcube.common.homelibrary.home.exceptions.InternalErrorException;
 import org.gcube.common.homelibrary.home.workspace.WorkspaceItem;
 import org.gcube.common.homelibrary.home.workspace.folder.items.GCubeItem;
@@ -41,6 +51,8 @@ import org.gcube.vomanagement.usermanagement.model.GCubeRole;
 import org.gcube.vomanagement.usermanagement.model.GatewayRolesNames;
 import org.slf4j.LoggerFactory;
 
+import com.fasterxml.jackson.databind.ObjectMapper;
+
 import eu.trentorise.opendata.jackan.model.CkanOrganization;
 
 /**
@@ -51,6 +63,11 @@ public class Utils {
 
 	// Logger
 	private static final org.slf4j.Logger logger = LoggerFactory.getLogger(Utils.class);
+	private static final String APPLICATION_ID_CATALOGUE_MANAGER = "org.gcube.datacatalogue.ProductCatalogue"; 
+	private static final String NOTIFICATION_MESSAGE = "Dear members,<br>The product <b></em>PRODUCT_TITLE</em></b> has been just published by <b>USER_FULLNAME</b>.<br>You can find it here: PRODUCT_URL <br>";
+	private static final String SOCIAL_SERVICE_APPLICATION_TOKEN = "2/tokens/generate-application-token/";
+	private static final String SOCIAL_SERVICE_WRITE_APPLICATION_POST = "2/posts/write-post-app/";
+	private static final String MEDIATYPE_JSON = "application/json";
 
 	/** Gets the gcube item properties.
 	 *
@@ -342,4 +359,121 @@ public class Utils {
 
 		return beans;
 	}
-}
+
+	/**
+	 * Send notification to vre members about the created product by writing a post.
+	 * @param productName the title of the product
+	 * @param productUrl the url of the product
+	 * @param hashtags a list of product's hashtags
+	 */
+	public static void writeProductPost(String productName, String productUrl, String userFullname, List<String> hashtags, boolean enablePostNotification){
+
+		// discover service endpoint for the social networking library
+		String currentScope = ScopeProvider.instance.get();
+		String tokenUser = SecurityTokenProvider.instance.get();
+
+		logger.info("Current scope for writeProductPost is " + currentScope + " and token is " + tokenUser.substring(0, 10) + "***************");
+		String basePath = new ServiceEndPointReaderSocial(currentScope).getBasePath();
+
+		if(basePath == null){
+
+			logger.error("Unable to write a post because there is no social networking service available");
+
+		}else{
+
+			try(CloseableHttpClient client = HttpClientBuilder.create().build();){
+
+				// ask token application
+				HttpPost postRequest = new HttpPost(basePath + SOCIAL_SERVICE_APPLICATION_TOKEN + "?gcube-token=" + tokenUser);
+				StringEntity input = new StringEntity("{\"app_id\":\"" + APPLICATION_ID_CATALOGUE_MANAGER + "\"}");
+				input.setContentType(MEDIATYPE_JSON);
+				postRequest.setEntity(input);
+				HttpResponse response = client.execute(postRequest);
+
+				logger.debug("Url is " + basePath + SOCIAL_SERVICE_APPLICATION_TOKEN + "?gcube-token=" + tokenUser);
+
+				if (response.getStatusLine().getStatusCode() != 201) {
+					throw new RuntimeException("Failed to retrieve application token : HTTP error code : "
+							+ response.getStatusLine().getStatusCode());
+				}else{
+
+					Map<String, Object> mapResponseGeneratedToken = getResponseEntityAsJSON(response);
+					boolean successGeneratedToken = (boolean)mapResponseGeneratedToken.get("success");
+					if(!successGeneratedToken){
+
+						throw new RuntimeException("Failed to generate the token for the application!"
+								+ " Error message is " + mapResponseGeneratedToken.get("message"));
+
+					}else{
+
+						String applicationToken = (String)mapResponseGeneratedToken.get("result");
+
+						// replace
+						String message  = NOTIFICATION_MESSAGE.replace("PRODUCT_TITLE", productName).replace("PRODUCT_URL", productUrl).replace("USER_FULLNAME", userFullname);
+
+						if(hashtags != null && !hashtags.isEmpty())
+							for (String hashtag : hashtags) {
+								String modifiedHashtag = hashtag.replaceAll(" ", "_").replace("_+", "_");
+								if(modifiedHashtag.endsWith("_"))
+									modifiedHashtag = modifiedHashtag.substring(0, modifiedHashtag.length() - 1);
+								message += " #" + modifiedHashtag; // ckan accepts tag with empty spaces, we don't
+							}
+
+						logger.info("The post that is going to be written is -> " + message);
+						postRequest = new HttpPost(basePath + SOCIAL_SERVICE_WRITE_APPLICATION_POST + "?gcube-token=" + applicationToken);
+						input = new StringEntity("{\"text\":\"" + message + "\", \"enable_notification\" : "+ enablePostNotification+ "}");
+						input.setContentType(MEDIATYPE_JSON);
+						postRequest.setEntity(input);
+						response = client.execute(postRequest);
+
+						Map<String, Object> mapResponseWritePost = getResponseEntityAsJSON(response);
+
+						if (response.getStatusLine().getStatusCode() != 201)
+							throw new RuntimeException("Failed to write application post : HTTP error code : "
+									+ response.getStatusLine().getStatusCode() + mapResponseWritePost.get("message"));
+					}
+
+				}
+
+			}catch(Exception e){
+				logger.error("Failed to create a post", e);
+			}
+		}
+	}
+
+	/**
+	 * Convert the json response to a map
+	 * @param response
+	 * @return
+	 */
+	public static Map<String, Object> getResponseEntityAsJSON(HttpResponse response){
+
+		Map<String, Object> toReturn = null;
+		HttpEntity entity = response.getEntity();
+
+		if (entity != null) {
+			try {
+				toReturn = new HashMap<String, Object>();
+				InputStream is = entity.getContent();
+				BufferedReader reader = new BufferedReader(new InputStreamReader(is));
+				StringBuilder sb = new StringBuilder();
+
+				String line = null;
+
+				while ((line = reader.readLine()) != null) {
+					sb.append(line + "\n");
+				}
+
+				logger.debug("Response as string is " + sb.toString());
+				ObjectMapper objectMapper = new ObjectMapper();
+				toReturn = objectMapper.readValue(sb.toString(), HashMap.class);
+				logger.debug("Map is " + toReturn);
+
+			}catch(Exception e){
+				logger.error("Failed to read json object", e);
+			}
+		}
+
+		return toReturn;
+	}
+}
\ No newline at end of file
